/*  Misfit Model 3D
 * 
 *  Copyright (c) 2004-2005 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 *  See the COPYING file for full license text.
 */


#include "texwin.h"
#include "texframe.h"
#include "texwidget.h"
#include "model.h"
#include "texture.h"
#include "texmgr.h"
#include "log.h"
#include "rgbawin.h"
#include "valuewin.h"
#include "decalmgr.h"
#include "msg.h"
#include "3dmprefs.h"
#include "helpwin.h"

#include "mq3compat.h"

#include <qcombobox.h>
#include <qpushbutton.h>
#include <list>
#include <string>
#include <qinputdialog.h>

using std::list;
using std::string;
TextureWindow::TextureWindow( Model * model, QWidget * parent, const char * name )
   : TextureWindowBase( parent, name, true, WDestructiveClose ),
     m_accel( new QAccel(this) ),
     m_model( model ),
     m_rgba( NULL ),
     m_lightType( 0 )
{
   m_textureFrame->setModel( model );

   m_accel->insertItem( Key_F1, 0 );
   connect( m_accel, SIGNAL(activated(int)), this, SLOT(helpNowEvent(int)) );

   int count = m_model->getTextureCount();

   for ( int t = 0; t < count; t++ )
   {
      //Texture * texture = m_model->getTextureData( t );
      //if ( texture )
      {
         m_textureComboBox->insertItem( m_model->getTextureName(t), t+1 );
      }
   }

   int textureId = -1;
   list<int> triangles = m_model->getSelectedTriangles();
   if ( triangles.size() > 0 )
   {
      textureId = m_model->getGroupTextureId( m_model->getTriangleGroup( triangles.front() ) );
   }

   m_textureComboBox->setCurrentItem( textureId + 1 );
   textureChangedEvent( textureId + 1 );
}

TextureWindow::~TextureWindow()
{
}

void TextureWindow::helpNowEvent( int id )
{
   HelpWin * win = new HelpWin( "olh_texturewin.html", true );
   win->show();
}

void TextureWindow::newTextureClickedEvent()
{
   list<string> formats = TextureManager::getInstance()->getAllReadTypes();

   QString formatsStr = "All Supported Formats (";

   list<string>::iterator it = formats.begin();
   while(  it != formats.end() )
   {
      formatsStr += QString( (*it).c_str() );

      it++;

      if ( it != formats.end() )
      {
         formatsStr += " ";
      }
   }

   formatsStr += ")";

   QString dir = g_prefs( "ui_texture_dir" );
   if ( dir.isEmpty() )
   {
      dir = ".";
   }

   QFileDialog d(dir, formatsStr, NULL, "", true );

   d.setCaption( "Open texture image" );
   d.addFilter( "All Files (*)" );
   d.setSelectedFilter( formatsStr );

   if ( QDialog::Accepted == d.exec() )
   {
      std::string file = d.selectedFile().latin1();
      g_prefs( "ui_texture_dir" ) = d.dir()->absPath();

      Texture * tex = TextureManager::getInstance()->getTexture( file.c_str() );
      if ( tex )
      {
         int num = m_model->addTexture( tex );
         m_textureComboBox->insertItem( m_model->getTextureName(num), num+1 );
         m_textureComboBox->setCurrentItem( num+1 );
         textureChangedEvent( num+1 );
         log_debug( "added %s as %d\n", file.c_str(), num );
      }
      else
      {
         QString err = tr(file.c_str()) + "\n";
         Texture::Error e = TextureManager::getInstance()->getLastError();
         if ( e != Texture::ERROR_NONE )
         {
            err += Texture::errorToString( e );
         }
         else
         {
            err += "Could not open file";
         }
         msg_error( err.latin1() );
      }
   }
}

void TextureWindow::newColorClickedEvent()
{
   bool ok = false;
   QString name = QInputDialog::getText( "Color Material", "Enter new material name:", QLineEdit::Normal, "", &ok );

   if ( ok )
   {
      int num = m_model->addColorMaterial( name.latin1() );
      m_textureComboBox->insertItem( m_model->getTextureName(num), num+1 );
      m_textureComboBox->setCurrentItem( num+1 );
      textureChangedEvent( num+1 );
      log_debug( "added %s as %d\n", name.latin1(), num );
   }
}

void TextureWindow::renameClickedEvent()
{
   int textureId = m_textureComboBox->currentItem() - 1;

   if ( textureId >= 0 )
   {
      bool ok = false;
      QString name = QInputDialog::getText( "Rename texture", "Enter new texture name:", QLineEdit::Normal, m_model->getTextureName( textureId ), &ok );

      if ( ok )
      {
         m_model->setTextureName( textureId, name.latin1() );
         m_textureComboBox->changeItem( name, textureId + 1 );
      }
   }
}

void TextureWindow::deleteClickedEvent()
{
   int id = m_textureComboBox->currentItem();
   if ( id > 0 )
   {
      m_model->deleteTexture( id - 1 );
      m_textureComboBox->removeItem( id );
      m_textureComboBox->setCurrentItem( 0 );
      textureChangedEvent( 0 );
   }
}

void TextureWindow::clampSChangedEvent( int index )
{
   int id = m_textureComboBox->currentItem() - 1;

   if ( id >= 0 )
   {
      m_model->setTextureSClamp( id, (index == 1) ? true : false );
      m_textureFrame->getTextureWidget()->setSClamp( (index == 1) ? true : false );
   }
}

void TextureWindow::clampTChangedEvent( int index )
{
   int id = m_textureComboBox->currentItem() - 1;

   if ( id >= 0 )
   {
      m_model->setTextureTClamp( id, (index == 1) ? true : false );
      m_textureFrame->getTextureWidget()->setTClamp( (index == 1) ? true : false );
   }
}

void TextureWindow::ambientClickedEvent()
{
   int id = (unsigned) m_textureComboBox->currentItem() - 1;
   float val[4];

   if ( m_model->getTextureAmbient( id, val) )
   {
      RgbaWin rgba;

      m_lightType = 0;
      m_rgba = &rgba;
      connect( &rgba, SIGNAL(valuesChanged()), this, SLOT(updateEvent()) );

      rgba.setLabel( "Ambient light" );

      rgba.setRed(   val[0] );
      rgba.setGreen( val[1] );
      rgba.setBlue(  val[2] );
      rgba.setAlpha( val[3] );

      if ( rgba.exec() )
      {
         val[0] = rgba.getRed();
         val[1] = rgba.getGreen();
         val[2] = rgba.getBlue();
         val[3] = rgba.getAlpha();

         m_model->setTextureAmbient( id, val );
         DecalManager::getInstance()->modelUpdated( m_model );
      }
      m_rgba = NULL;
   }
   else
   {
      log_error( "could not get ambient values for %d\n", id );
   }
}

void TextureWindow::diffuseClickedEvent()
{
   int id = (unsigned) m_textureComboBox->currentItem() - 1;
   float val[4];

   if ( m_model->getTextureDiffuse( id, val) )
   {
      RgbaWin rgba;

      m_lightType = 1;
      m_rgba = &rgba;
      connect( &rgba, SIGNAL(valuesChanged()), this, SLOT(updateEvent()) );

      rgba.setLabel( "Diffuse light" );

      rgba.setRed(   val[0] );
      rgba.setGreen( val[1] );
      rgba.setBlue(  val[2] );
      rgba.setAlpha( val[3] );

      if ( rgba.exec() )
      {
         val[0] = rgba.getRed();
         val[1] = rgba.getGreen();
         val[2] = rgba.getBlue();
         val[3] = rgba.getAlpha();

         m_model->setTextureDiffuse( id, val );
         DecalManager::getInstance()->modelUpdated( m_model );
      }
      m_rgba = NULL;
   }
   else
   {
      log_error( "could not get diffuse values for %d\n", id );
   }
}

void TextureWindow::specularClickedEvent()
{
   int id = (unsigned) m_textureComboBox->currentItem() - 1;
   float val[4];

   if ( m_model->getTextureSpecular( id, val) )
   {
      RgbaWin rgba;

      m_lightType = 2;
      m_rgba = &rgba;
      connect( &rgba, SIGNAL(valuesChanged()), this, SLOT(updateEvent()) );

      rgba.setLabel( "Specular light" );

      rgba.setRed(   val[0] );
      rgba.setGreen( val[1] );
      rgba.setBlue(  val[2] );
      rgba.setAlpha( val[3] );

      if ( rgba.exec() )
      {
         val[0] = rgba.getRed();
         val[1] = rgba.getGreen();
         val[2] = rgba.getBlue();
         val[3] = rgba.getAlpha();

         m_model->setTextureSpecular( id, val );
         DecalManager::getInstance()->modelUpdated( m_model );
      }
      m_rgba = NULL;
   }
   else
   {
      log_error( "could not get specular values for %d\n", id );
   }
}

void TextureWindow::emissiveClickedEvent()
{
   int id = (unsigned) m_textureComboBox->currentItem() - 1;
   float val[4];

   if ( m_model->getTextureEmissive( id, val) )
   {
      RgbaWin rgba;

      m_lightType = 3;
      m_rgba = &rgba;
      connect( &rgba, SIGNAL(valuesChanged()), this, SLOT(updateEvent()) );

      rgba.setLabel( "Emissive light" );

      rgba.setRed(   val[0] );
      rgba.setGreen( val[1] );
      rgba.setBlue(  val[2] );
      rgba.setAlpha( val[3] );

      if ( rgba.exec() )
      {
         val[0] = rgba.getRed();
         val[1] = rgba.getGreen();
         val[2] = rgba.getBlue();
         val[3] = rgba.getAlpha();

         m_model->setTextureEmissive( id, val );
         DecalManager::getInstance()->modelUpdated( m_model );
      }
      m_rgba = NULL;
   }
   else
   {
      log_error( "could not get emissive values for %d\n", id );
   }
}

void TextureWindow::shininessClickedEvent()
{
   int id = (unsigned) m_textureComboBox->currentItem() - 1;
   float val;

   if ( m_model->getTextureShininess( id, val) )
   {
      ValueWin value;

      value.setLabel( "Shininess" );

      value.setValue( val );

      if ( value.exec() )
      {
         val = value.getValue();
         m_model->setTextureShininess( id, val );
         DecalManager::getInstance()->modelUpdated( m_model );
      }
   }
   else
   {
      log_error( "could not get shininess values for %d\n", id );
   }
}

void TextureWindow::textureChangedEvent( int id )
{
   m_textureFrame->textureChangedEvent( id );

   id = id - 1;

   if ( id >= 0 )
   {
      m_ambientButton->setEnabled( true );
      m_diffuseButton->setEnabled( true );
      m_specularButton->setEnabled( true );
      m_emissiveButton->setEnabled( true );
      m_shininessButton->setEnabled( true );
      m_deleteButton->setEnabled( true );
      m_renameButton->setEnabled( true );
      m_sClamp->setEnabled( true );
      m_tClamp->setEnabled( true );

      m_sClamp->setCurrentItem( (m_model->getTextureSClamp( id )) ? 1 : 0 );
      m_tClamp->setCurrentItem( (m_model->getTextureTClamp( id )) ? 1 : 0 );
      m_textureFrame->getTextureWidget()->setSClamp( m_model->getTextureSClamp( id ) );
      m_textureFrame->getTextureWidget()->setTClamp( m_model->getTextureTClamp( id ) );
   }
   else
   {
      m_ambientButton->setEnabled( false );
      m_diffuseButton->setEnabled( false );
      m_specularButton->setEnabled( false );
      m_emissiveButton->setEnabled( false );
      m_shininessButton->setEnabled( false );
      m_deleteButton->setEnabled( false );
      m_renameButton->setEnabled( false );
      m_sClamp->setEnabled( false );
      m_tClamp->setEnabled( false );
   }
}

void TextureWindow::updateEvent()
{
   if ( m_rgba )
   {
      int id = (unsigned) m_textureComboBox->currentItem() - 1;

      float val[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
      val[0] = m_rgba->getRed();
      val[1] = m_rgba->getGreen();
      val[2] = m_rgba->getBlue();
      val[3] = m_rgba->getAlpha();

      switch ( m_lightType )
      {
         case 0:
            m_model->setTextureAmbient( id, val );
            break;
         case 1:
            m_model->setTextureDiffuse( id, val );
            break;
         case 2:
            m_model->setTextureSpecular( id, val );
            break;
         case 3:
            m_model->setTextureEmissive( id, val );
            break;
         default:
            break;
      }
      DecalManager::getInstance()->modelUpdated( m_model );
      m_textureFrame->getTextureWidget()->updateGL();
   }
}

void TextureWindow::accept()
{
   m_model->operationComplete( "Texture changes" );
   TextureWindowBase::accept();
   DecalManager::getInstance()->modelUpdated( m_model );
}

void TextureWindow::reject()
{
   m_model->undoCurrent();
   TextureWindowBase::reject();
}

